//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 一次元テクスチャ座標ヘッダ
 * @author Junpee
 */

#ifndef TEX_COORD_1_H_
#define TEX_COORD_1_H_

#include <Core/System/Math.h>
#include <Core/Primitive/String.h>

namespace Lamp{

//------------------------------------------------------------------------------
/**
 * 一次元テクスチャ座標
 *
 * このクラスは継承しないで下さい。
 */
class TexCoord1{
public:
	//--------------------------------------------------------------------------
	// メンバ変数
	//--------------------------------------------------------------------------
	/// メンバ変数
	union{
		/// 各要素
		struct{
			/// U値
			float u;
		};

		/// 配列
		float array[1];
	};

	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// ゼロ座標
	static const TexCoord1 zero;

	/// 単位座標
	static const TexCoord1 unit;

	/// U単位座標
	static const TexCoord1 unitU;

	//--------------------------------------------------------------------------
	// コンストラクタ
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタは初期値の設定を行わないため値は不定です。
	 */
	inline TexCoord1(){}

	/**
	 * コンストラクタ
	 * @param sourceU Uの初期値
	 */
	inline TexCoord1(float sourceU) : u(sourceU){}

	/**
	 * コンストラクタ
	 * @param source 初期値配列
	 */
	inline explicit TexCoord1(const float* const source) : u(source[0]){}

	//--------------------------------------------------------------------------
	// 値の設定
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param sourceU Uの設定値
	 */
	inline void set(float sourceU){ u = sourceU; }

	/**
	 * 値の設定
	 * @param source 設定値配列
	 */
	inline void set(const float* const source){ u = source[0]; }

	//--------------------------------------------------------------------------
	// 演算
	//--------------------------------------------------------------------------
	/**
	 * 加算
	 * @param addCoord 加算する一次元テクスチャ座標
	 * @return 加算された一次元テクスチャ座標
	 */
	inline TexCoord1 operator +(const TexCoord1& addCoord) const{
		return TexCoord1(u + addCoord.u);
	}

	/**
	 * 減算
	 * @param subCoord 減算する一次元テクスチャ座標
	 * @return 減算された一次元テクスチャ座標
	 */
	inline TexCoord1 operator -(const TexCoord1& subCoord) const{
		return TexCoord1(u - subCoord.u);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された一次元テクスチャ座標
	 */
	inline TexCoord1 operator *(float mulValue) const{
		return TexCoord1(u * mulValue);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @param mulCoord 乗算する一次元テクスチャ座標
	 * @return 乗算された一次元テクスチャ座標
	 */
	inline friend TexCoord1 operator *(
		float mulValue, const TexCoord1& mulCoord){
		return TexCoord1(mulCoord.u * mulValue);
	}

	/**
	 * +演算子
	 * @return 一次元テクスチャ座標のコピー
	 */
	inline TexCoord1 operator +() const{ return *this; }

	/**
	 * -演算子
	 * @return 値の符号が反転した一次元テクスチャ座標
	 */
	inline TexCoord1 operator -() const{ return TexCoord1(-u); }

	//--------------------------------------------------------------------------
	// 代入演算
	//--------------------------------------------------------------------------
	/**
	 * 代入加算
	 * @param addCoord 加算する一次元テクスチャ座標
	 * @return 加算された一次元テクスチャ座標
	 */
	inline TexCoord1& operator +=(const TexCoord1& addCoord){
		u += addCoord.u;
		return *this;
	}

	/**
	 * 代入減算
	 * @param subCoord 減算する一次元テクスチャ座標
	 * @return 減算された一次元テクスチャ座標
	 */
	inline TexCoord1& operator -=(const TexCoord1& subCoord){
		u -= subCoord.u;
		return *this;
	}

	/**
	 * 代入乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された一次元テクスチャ座標
	 */
	inline TexCoord1& operator *=(float mulValue){
		u *= mulValue;
		return *this;
	}

	//--------------------------------------------------------------------------
	// 論理演算
	//--------------------------------------------------------------------------
	/**
	 * 一次元テクスチャ座標が同じかどうか
	 * @param target 比較する一次元テクスチャ座標
	 * @return 同じ値であればtrueを返す
	 */
	inline bool operator ==(const TexCoord1& target) const{
		return (u == target.u);
	}

	/**
	 * 一次元テクスチャ座標が同じかどうか
	 * @param target 比較する一次元テクスチャ座標
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じ値であればtrueを返す
	 */
	inline bool epsilonEquals(
		const TexCoord1& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (Math::abs(u - target.u) <= epsilon);
	}

	/**
	 * 一次元テクスチャ座標が同じでないかどうか
	 * @param target 比較する一次元テクスチャ座標
	 * @return 同じでない値であればtrueを返す
	 */
	inline bool operator !=(const TexCoord1& target) const{
		return (u != target.u);
	}

	/**
	 * 一次元テクスチャ座標が同じでないかどうか
	 * @param target 比較する一次元テクスチャ座標
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じでない値であればtrueを返す
	 */
	inline bool notEpsilonEquals(
		const TexCoord1& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (Math::abs(u - target.u) > epsilon);
	}

	//--------------------------------------------------------------------------
	// その他
	//--------------------------------------------------------------------------
	/**
	 * 文字列化
	 * @return 一次元テクスチャ座標の文字列表記
	 */
	inline String toString() const{
		String returnString;
		returnString.format("( %.8f )", u);
		return returnString;
	}

	//--------------------------------------------------------------------------
private:

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of TEX_COORD_1_H_
//------------------------------------------------------------------------------
